unit QuizzClass;

interface

uses QuizzFile;

type
  TOnAfficherResultat = procedure( Reponse: Integer ) of object;
  TOnAfficherResultats = procedure( NombreReponseBonne, NombreReponseMauvaise: Integer ) of object;
  TOnAfficherQuestion = procedure( Question: TQuestion ) of object;

  TQuizzClass = class
  private
    FQuizz: TWordFile;
    FNiveau: Integer;

    FMauvaiseReponses: Integer;
    FQuestionIndex: Integer;
    FBonneReponses: Integer;
    FOnAfficherQuestion: TOnAfficherQuestion;
    FOnAfficherResultat: TOnAfficherResultat;
    FOnAfficherResultats: TOnAfficherResultats;

    procedure SetOnAfficherQuestion(const Value: TOnAfficherQuestion);
    procedure SetOnAfficherResultat(const Value: TOnAfficherResultat);
    procedure SetOnAfficherResultats(const Value: TOnAfficherResultats);
  public
    constructor Create( Quizz: TWordFile; Niveau: Integer );

    function Correction( ReponseQuestion: Integer ): Integer;
    procedure FirstQuestion;
    procedure NextQuestion;

    property QuestionIndex: Integer read FQuestionIndex;
    property BonneReponses: Integer read FBonneReponses;
    property MauvaiseReponses: Integer read FMauvaiseReponses;

    property OnAfficherResultat: TOnAfficherResultat read FOnAfficherResultat write SetOnAfficherResultat;
    property OnAfficherResultats: TOnAfficherResultats read FOnAfficherResultats write SetOnAfficherResultats;
    property OnAfficherQuestion: TOnAfficherQuestion read FOnAfficherQuestion write SetOnAfficherQuestion;
  end;

implementation

{ TQuizzClass }

function TQuizzClass.Correction(ReponseQuestion: Integer): Integer;
var Reponse: Integer;
begin
  Reponse := ( ( FQuizz.Niveaux[ FNiveau ] as TNiveau ).Questions[ FQuestionIndex ] as TQuestion ).Reponse;
  if( Reponse <> ReponseQuestion ) then
    Inc( FMauvaiseReponses )
  else
    Inc( FBonneReponses );

  Result := Reponse;

  if( Assigned(FOnAfficherResultat) ) then
    FOnAfficherResultat( Reponse );
end;

constructor TQuizzClass.Create(Quizz: TWordFile; Niveau: Integer);
begin
  FQuizz  := Quizz;
  FNiveau := Niveau;
end;

procedure TQuizzClass.FirstQuestion;
begin
  FQuestionIndex    := -1;
  FMauvaiseReponses := 0;
  FBonneReponses    := 0;

  NextQuestion;
end;

procedure TQuizzClass.NextQuestion;
begin
  Inc( FQuestionIndex );

  if( FQuestionIndex < ( FQuizz.Niveaux[ FNiveau ] as TNiveau ).Questions.Count ) then begin
    if( Assigned(FOnAfficherQuestion) ) then
      FOnAfficherQuestion( ( FQuizz.Niveaux[ FNiveau ] as TNiveau ).Questions[ FQuestionIndex ] as TQuestion );
  end else begin
    if( Assigned(FOnAfficherResultats) ) then
      FOnAfficherResultats( FBonneReponses, FMauvaiseReponses );
  end;
end;

procedure TQuizzClass.SetOnAfficherQuestion(const Value: TOnAfficherQuestion);
begin
  FOnAfficherQuestion := Value;
end;

procedure TQuizzClass.SetOnAfficherResultat(const Value: TOnAfficherResultat);
begin
  FOnAfficherResultat := Value;
end;

procedure TQuizzClass.SetOnAfficherResultats(const Value: TOnAfficherResultats);
begin
  FOnAfficherResultats := Value;
end;

end.
