unit QuizzFile;

interface

uses Contnrs, Graphics, Classes, IniFiles;

type
  TWordFile = class
  private
    FNiveaux: TObjectList;
    FMessages: TObjectList;
  public
    constructor Create(Fichier: TIniFile);
    destructor Destroy(); override;

    function MessageOf(NoteSur100: Integer): String;

    property Niveaux: TObjectList read FNiveaux;
    property Messages: TObjectList read FMessages;
  end;

  TMessage = class
  private
    FBorneMin: Integer;
    FMessageString: String;
    procedure SetBorneMin(const Value: Integer);
    procedure SetMessageString(const Value: String);
  public
    constructor Create(BorneMin: Integer; MessageString: String);

    property BorneMin: Integer read FBorneMin write SetBorneMin;
    property MessageString: String read FMessageString write SetMessageString;
  end;

  TQuestion = class;

  TNiveau = class
  private
    FNiveau: String;
    FQuestions: TObjectList;
    procedure SetNiveau(const Value: String);

    function CreateQuestion(Fichier: TIniFile; Niveau, Question: Integer) : TQuestion;
  public
    constructor Create(Fichier: TIniFile; Niveau: Integer);

    property Niveau: String read FNiveau write SetNiveau;
    property Questions: TObjectList read FQuestions;
  end;

  TQuestion = class
  private
    FReponse: Integer;
    FQuestionType: String;
    FQuestion: String;
    FNbReponse: Integer;
    FImage: TPicture;
    FIndice: String;
    procedure SetQuestion(const Value: String);
    procedure SetQuestionType(const Value: String);
    procedure SetReponse(const Value: Integer);
    procedure SetNbReponse(const Value: Integer);
    procedure SetImage(const Value: TPicture);
  protected
    function Balise( Niveau, Question : Integer ): String;
  public
    constructor Create(Fichier: TIniFile; Niveau, Question: Integer);

    property Image: TPicture read FImage write SetImage;
    property QuestionType: String read FQuestionType write SetQuestionType;
    property Question: String read FQuestion write SetQuestion;
    property NbReponse: Integer read FNbReponse write SetNbReponse;
    property Reponse: Integer read FReponse write SetReponse;
    property Indice: String read FIndice write FIndice;
  end;

  TQuestionString = class( TQuestion )
  private
    FReponses: TStringList;
  public
    constructor Create(Fichier: TIniFile; Niveau, Question: Integer);
    property Reponses: TStringList read FReponses;
  end;

  TQuestionPicture = class( TQuestion )
  private
    FPictures: TObjectList;
  public
    constructor Create(Fichier: TIniFile; Niveau, Question: Integer);

    property Pictures: TObjectList read FPictures;
  end;

implementation

uses SysUtils;

{ TQuestion }

function TQuestion.Balise(Niveau, Question: Integer): String;
begin
  Result := 'Niveau' + IntToStr(Niveau) + '_Question' + IntToStr(Question);
end;

constructor TQuestion.Create(Fichier: TIniFile; Niveau, Question: Integer);
var ImageName: String;
begin
  FReponse      := Fichier.ReadInteger( Balise( Niveau, Question ), 'Resultat', -1 );
  FQuestionType := Fichier.ReadString( Balise( Niveau, Question ), 'Type', 'String' );
  FQuestion     := Fichier.ReadString( Balise( Niveau, Question ), 'Question', 'Pas de question' );
  FNbReponse    := Fichier.ReadInteger( Balise( Niveau, Question ), 'Nombre de rponse', 0 );
  FIndice       := Fichier.ReadString( Balise( Niveau, Question ), 'Indice', '' );

  ImageName := Fichier.ReadString( Balise( Niveau, Question ), 'Image', '' );
  FImage    := TPicture.Create;
  try
    FImage.LoadFromFile( ImageName );
  except
    FImage := nil;
  end;

end;

procedure TQuestion.SetNbReponse(const Value: Integer);
begin
  FNbReponse := Value;
end;

procedure TQuestion.SetQuestion(const Value: String);
begin
  FQuestion := Value;
end;

procedure TQuestion.SetQuestionType(const Value: String);
begin
  FQuestionType := Value;
end;

procedure TQuestion.SetReponse(const Value: Integer);
begin
  FReponse := Value;
end;

procedure TQuestion.SetImage(const Value: TPicture);
begin
  FImage := Value;
end;

{ TQuestionString }

constructor TQuestionString.Create(Fichier: TIniFile; Niveau,
  Question: Integer);
var Reponse  : String;
    I        : Integer;
begin
  inherited Create(Fichier, Niveau, Question);

  FReponses := TStringList.Create;

  for I := 1 to NbReponse do begin
    Reponse := Fichier.ReadString( Balise( Niveau, Question ), 'Reponse' + IntToStr(I), 'No Response' );
    FReponses.Add( Reponse );
  end;
end;

{ TNiveau }

constructor TNiveau.Create(Fichier: TIniFile; Niveau: Integer);
var NbQuestion, I: Integer;
begin
  NbQuestion := Fichier.ReadInteger( 'Niveau' + IntToStr(Niveau), 'Nombre question', 1 );

  FQuestions := TObjectList.Create;
  for I := 1 to NbQuestion do begin
    FQuestions.Add( CreateQuestion( Fichier, Niveau, I ) );
  end;

  FNiveau := Fichier.ReadString( 'Level', IntToStr(Niveau), '' );
end;

function TNiveau.CreateQuestion(Fichier: TIniFile; Niveau,
  Question: Integer): TQuestion;
var TypeQuestion: String;
    Quest: TQuestion;
begin
  TypeQuestion := Fichier.ReadString( 'Niveau' + IntToStr(Niveau) + '_Question' + IntToStr(Question), 'Type', 'Texte' );
  if( TypeQuestion = 'Texte' ) then begin
    Quest := TQuestionString.Create( Fichier, Niveau, Question );
  end else begin
    Quest := TQuestionPicture.Create( Fichier, Niveau, Question );
  end;

  Result := Quest;
end;

procedure TNiveau.SetNiveau(const Value: String);
begin
  FNiveau := Value;
end;

{ TWordFile }

{ TMessage }

constructor TMessage.Create(BorneMin: Integer; MessageString: String);
begin
  FBorneMin := BorneMin;
  FMessageString := MessageString;
end;

procedure TMessage.SetBorneMin(const Value: Integer);
begin
  FBorneMin := Value;
end;

procedure TMessage.SetMessageString(const Value: String);
begin
  FMessageString := Value;
end;

{ TWordFile }

constructor TWordFile.Create(Fichier: TIniFile);
var Niveaux, Messages: TStringList;
    I, Borne         : Integer;
    MessageString    : String;
begin
  Niveaux := TStringList.Create;
  Fichier.ReadSectionValues( 'Level', Niveaux );

  FNiveaux := TObjectList.Create;
  for I := 1 to Niveaux.Count do
    FNiveaux.Add( TNiveau.Create( Fichier, I ) );

  Messages := TStringList.Create;
  Fichier.ReadSectionValues( 'Messages', Messages );

  FMessages := TObjectList.Create;
  for I := 0 to Messages.Count - 1 do begin
    Borne := StrToInt(Messages.Names[I]);
    MessageString := Messages.Values[ IntToStr(Borne) ];

    FMessages.Add( TMessage.Create( Borne, MessageString ) );
  end;
end;

destructor TWordFile.Destroy;
begin
  Niveaux.Free;
  Messages.Free;

  inherited;
end;

function TWordFile.MessageOf(NoteSur100: Integer): String;
var I: Integer;
begin
  // ATTENTION : IL EST NECESSAIRE QUE LES MESSAGES SOIT DANS L'ORDRE CROISSANT DE NOTE
  for I := FMessages.Count - 1 downto 0 do begin
    if( NoteSur100 >= (FMessages[I] as TMessage).BorneMin ) then begin
      Result := (FMessages[I] as TMessage).MessageString;
      Break;
    end;
  end;
end;

{ TQuestionPicture }

constructor TQuestionPicture.Create(Fichier: TIniFile; Niveau,
  Question: Integer);
var ImageName    : String;
    I            : Integer;
    NbReponseErr : Integer;
    Picture      : TPicture;
begin
  inherited Create(Fichier, Niveau, Question);

  FPictures := TObjectList.Create;
  NbReponseErr := 0;

  for I := 1 to NbReponse do begin
    ImageName := Fichier.ReadString( Balise( Niveau, Question ), 'Image' + IntToStr(I), '' );
    Picture   := TPicture.Create;
    try
      Picture.LoadFromFile( ImageName );
      FPictures.Add( Picture );
    except
      Inc(NbReponseErr);
    end;
  end;
  Dec(FNbReponse, NbReponseErr);
end;

end.
